﻿// <copyright>
//   Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>

/*
 *   DF changes
 *     Was called "BigShelfService.metadata.cs but has nothing to do with service so
 *     renamed as "BigShelf.metadata.cs" and moved from Services to Models
 *     
 *     Added [Association(...)] attributes;
 *     ToDo: teach RIA Metadata pipeline to understand DevForce relations or
 *           teach DF to generate these attributes
 *     Most of RIA wizard generated is irrelevant; removed the dross  (optional)
 *     The actual return values of nav props doesn't matter; use IList<T> (optional)
 */
// ReSharper disable RedundantNameQualifier

namespace BigShelf.Models 
{
    using System;
    using System.ComponentModel.DataAnnotations;
    using System.ServiceModel.DomainServices.Server;
    using System.Collections.Generic;

    // The MetadataTypeAttribute identifies BookMetadata as the class
    // that carries additional metadata for the Book class.
    [MetadataTypeAttribute(typeof(Book.BookMetadata))]
    public partial class Book
    {
        // This class allows you to attach custom attributes to properties
        // of the Book class.
        //
        // For example, the following marks the Xyz property as a
        // required property and specifies the format for valid values:
        //    [Required]
        //    [RegularExpression("[A-Z][A-Za-z0-9]*")]
        //    [StringLength(32)]
        //    public string Xyz { get; set; }
        internal sealed class BookMetadata
        {
            // Metadata classes are not meant to be instantiated.
            private BookMetadata(){}

            [Association("CategoryName_Book", "CategoryId", "Id", IsForeignKey = true)]
            public CategoryName CategoryName { get; set; }

            [Association("Book_FlaggedBook", "Id", "BookId")]
            public IList<FlaggedBook> FlaggedBooks { get; set; }
        }
    }

    [MetadataTypeAttribute(typeof(Category.CategoryMetadata))]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.StyleCop.CSharp.MaintainabilityRules", "SA1402:FileMayOnlyContainASingleClass", Justification = "Standard RIA Services project format")]
    public partial class Category
    {
        internal sealed class CategoryMetadata
        {
            private CategoryMetadata(){}

            [Include]
            [Association("CategoryName_Category", "CategoryId", "Id", IsForeignKey = true)]
            public CategoryName CategoryName { get; set; }

            [Association("Profile_Category", "ProfileId", "Id", IsForeignKey = true)]
            public Profile Profile { get; set; }
        }
    }

    [MetadataTypeAttribute(typeof(CategoryName.CategoryNameMetadata))]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.StyleCop.CSharp.MaintainabilityRules", "SA1402:FileMayOnlyContainASingleClass", Justification = "Standard RIA Services project format")]
    public partial class CategoryName
    {
        internal sealed class CategoryNameMetadata
        {
            private CategoryNameMetadata(){}

            [Association("CategoryName_Book", "Id", "CategoryId")]
            public IList<Book> Books { get; set; }

            [Association("CategoryName_Category", "Id", "CategoryId")]
            public IList<Category> Categories { get; set; }
        }
    }

    [MetadataTypeAttribute(typeof(FlaggedBook.FlaggedBookMetadata))]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.StyleCop.CSharp.MaintainabilityRules", "SA1402:FileMayOnlyContainASingleClass", Justification = "Standard RIA Services project format")]
    public partial class FlaggedBook
    {
        internal sealed class FlaggedBookMetadata
        {
            private FlaggedBookMetadata(){}

            [Include]
            [Association("Book_FlaggedBook", "BookId", "Id", IsForeignKey = true)]
            public Book Book { get; set; }

            [Association("Profile_FlaggedBook", "ProfileId", "Id", IsForeignKey = true)]
            public Profile Profile { get; set; }
        }
    }

    [MetadataTypeAttribute(typeof(Friend.FriendMetadata))]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.StyleCop.CSharp.MaintainabilityRules", "SA1402:FileMayOnlyContainASingleClass", Justification = "Standard RIA Services project format")]
    public partial class Friend
    {
        internal sealed class FriendMetadata
        {
            private FriendMetadata(){}

            [Include]
            [Association("Friend_Friend", "FriendId", "Id", IsForeignKey = true)]
            public Profile FriendProfile { get; set; }

            [Association("Profile_Friend", "ProfileId", "Id", IsForeignKey = true)]
            public Profile Profile { get; set; }
        }
    }

    [MetadataTypeAttribute(typeof(Profile.ProfileMetadata))]
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.StyleCop.CSharp.MaintainabilityRules", "SA1402:FileMayOnlyContainASingleClass", Justification = "Standard RIA Services project format")]
    public partial class Profile
    {

        internal sealed class ProfileMetadata
        {
            private ProfileMetadata(){}

            [Include]
            [Association("Profile_Category", "Id", "ProfileId")]
            public IList<Category> Categories { get; set; }

            [Include]
            [Association("Profile_FlaggedBook", "Id", "ProfileId")]
            public IList<FlaggedBook> FlaggedBooks { get; set; }

            [Include]
            [Association("Profile_Friend", "Id", "ProfileId")]
            public IList<Friend> Friends { get; set; }

            [Required]
            public string Name { get; set; }

            [Required, DataType(DataType.EmailAddress)]
            public string EmailAddress { get; set; }
        }
     }
}
