using System.ServiceModel.DomainServices.Server;
using System.Linq;
using IdeaBlade.EntityModel;
/*
 *   DF variation on LinkToEntitiesDomainService<T>. 
 *     using IdeaBlade.EntityModel
 *     ObjectContext -> Manager
 *     Added Query override so serialization of nav props doesn't load them
 *     Added insert/update/delete helpers for cleaner DomainService authoring
 */
namespace BigShelf
{
    public abstract class DevForceDomainService<TManager> : DomainService where TManager : EntityManager
    {
        private TManager _manager;

        public override void Initialize(DomainServiceContext context)
        {
            base.Initialize(context);
            if (_manager == null) _manager = CreateManager();
        }

        protected virtual TManager CreateManager()
        {
            return (TManager) System.Activator.CreateInstance(typeof(TManager), 
                new object[]
                {
                    true, //should connect
                    null, // no data source extension
                    EntityServiceOption.UseDefaultService,
                    null //no composition context
                });
        }

        /// <summary>
        /// DevForce EntityManager is sort of like the Entity Framework ObjectContext
        /// </summary>
        protected internal TManager Manager
        {
            get { return _manager; }
            set { _manager = value; }
        }

        protected override int Count<T>(IQueryable<T> query)
        {
            return query.Count();
        }

        protected override bool PersistChangeSet()
        {
            Manager.SaveChanges();
            return true;
        }


        public override System.Collections.IEnumerable Query(QueryDescription queryDescription, out System.Collections.Generic.IEnumerable<System.ComponentModel.DataAnnotations.ValidationResult> validationErrors, out int totalCount)
        {
            Manager.DefaultQueryStrategy = DefaultQueryStrategy;
            var result = base.Query(queryDescription, out validationErrors, out totalCount);
            // prevent lazy loading after query completed.
            Manager.DefaultQueryStrategy = QueryStrategy.CacheOnly;
            return result;
        }

        protected virtual QueryStrategy DefaultQueryStrategy
        {
            get
            {
                return QueryStrategy.Normal;
            }
        }

        #region Insert/Update/Delete helpers

        protected virtual void InsertEntity(IEntity entity)
        {
            if (entity != null) Manager.AddEntity(entity);
        }

        protected virtual void UpdateEntity(IEntity entity)
        {
            if (entity != null) Manager.AttachEntity(entity, EntityState.Modified);
        }

        protected virtual void DeleteEntity(IEntity entity)
        {
            if (entity != null)
            {
                Manager.AttachEntity(entity);
                entity.EntityAspect.Delete();
            }
        }

        #endregion

    }
}